#!/usr/bin/env python3

def test_acgc():
    import acgc 

def test_erroranalysis():
    import acgc.erroranalysis

def test_figstyle():
    import acgc.figstyle

def test_gctools():
    import acgc.gctools

def test_hytools():
    import acgc.hytools

def test_icartt():
    import acgc.icartt

def test_igra():
    import acgc.igra

def test_map_scalebar():
    import acgc.map_scalebar

def test_mettools():
    import acgc.mettools

def test_modetools():
    import acgc.modetools

def test_nctools():
    import acgc.nctools

def test_solar():
    import acgc.solar

def test_stats():
    import acgc.stats

def test_timetools():
    import acgc.timetools
